import {
  select,
  selectAll,
  scaleLog,
  scaleLinear,
  scaleOrdinal,
  schemePastel1,
  axisBottom,
  format,
  axisLeft,
  json,
  interval,
} from 'd3';

import {dropDownMenu} from '/dropDown.js'
// import * as dfd from "danfojs";

const margin = {
  left: 100,
  right: 10,
  top: 10,
  bottom: 100,
};

const width = 800 - margin.left - margin.right;
const height = 500 - margin.top - margin.bottom;


dropDownMenu();

const svg = select('body')
  .append('svg')
  .attr(
    'width',
    width + margin.left + margin.right
  )
  .attr(
    'height',
    height + margin.top + margin.bottom
  );


const g = svg
  .append('g')
  .attr(
    'transform',
    `translate(${margin.left}, ${margin.top})`
  );

let time = 0;
let selection;
let int; 

const x = scaleLog()
  .base(10)
  .range([0, width])
  .domain([142, 150000]);

const y = scaleLinear()
  .range([height, 0])
  .domain([0, 90]);

const area = scaleLinear()
  .range([25 * Math.PI, 1500 * Math.PI])
  .domain([2000, 1400000000]);



const continentColor = scaleOrdinal(
  schemePastel1
);

const xLabel = g
  .append('text')
  .attr('y', height + 50)
  .attr('x', width / 2)
  .attr('font-size', '20px')
  .attr('text-anchor', 'middle')
  .text('GDP Per Capita ($)');

const yLabel = g
  .append('text')
  .attr('transform', 'rotate(-90)')
  .attr('y', -40)
  .attr('x', -170)
  .attr('font-size', '20px')
  .attr('text-anchor', 'middle')
  .text('Life Expectancy (Years)');

const timeLabel = g
  .append('text')
  .attr('y', height - 10)
  .attr('x', width - 40)
  .attr('font-size', '40px')
  .attr('opacity', '0.4')
  .attr('text-anchor', 'middle')
	.attr('id', 'timelabel')
  .text('1800');



const xAxis = g
  .append('g')
  .attr('transform', `translate(0, ${height})`)
  .call(
    axisBottom(x)
      .tickValues([400, 4000, 40000])
      .tickFormat(format('$.0f'))
  );

const yAxis = g.append('g').call(axisLeft(y));

const continents = [
  'europe',
  'asia',
  'americas',
  'africa',
];

const legend = g
  .append('g')
  .attr(
    'transform',
    `translate(${width - 10}, ${height - 125})`
  );



continents.forEach((continent, i) => {
  const legendRow = legend
    .append('g')
    .attr('transform', `translate(0, ${i * 20})`);

  legendRow
    .append('rect')
    .attr('width', 10)
    .attr('height', 10)
    .attr('fill', continentColor(continent));

  legendRow
    .append('text')
  	.style('text-transform', 'capitalize')
    .attr('x', -10)
    .attr('y', 10)
    .attr('text-anchor', 'end')
    .text(continent)
  	
});





//main function
const main = async () => {
  const data = await json(
    'https://raw.githubusercontent.com/adamjanes/udemy-d3/master/05/5.10.1/data/data.json'
  );
  
  const formattedData = data.map((year) => {
    return year.countries.filter(
      (country) =>
        country.income && country.life_exp
    );
  });
  
   let filteredData = (selection) => {return formattedData.map((d) => d.filter((c) => selection === undefined ? true : selection == 'All' ? true : c.continent == selection.toLowerCase()))}
   console.log(filteredData(selection));
  const tooltip = 
    select('body')
    .append('div')
    .attr('class', 'tooltip');

  tooltip
    .append('rect')
    .attr('width', 60)
    .attr('height', 20)
    .attr('fill', 'none')
    .style('opacity', 0.5);

  tooltip
    .append('text')
    .attr('x', 30)
    .attr('dy', '1.2em')
    .style('text-anchor', 'middle')
    .attr('font-size', '12px')
    .attr('font-weight', 'bold');

  function update(selection) {
    const t = svg.transition().duration(150);
     
      
    const circles = g
      .selectAll('circle')
      .data(filteredData(selection)[time], (d) => d.country)
      .join(update(selection))
      .attr('fill', d =>
        	continentColor(d.continent)
      )
    .on('mouseover', function(event, d){
      event.stopPropagation();
     	tooltip.style('display', 'inline-block')
       .html(d.country + '<br/>' + `Population: ${d.population}` + '<br/>' + `Life Exp. : ${d.life_exp}`)
      .style('left', event.pageX - 50 + 'px')
      .style('top', event.pageY - 70 + 'px')
    })
    .on('mouseout', function(event, d){
      event.stopPropagation();
    			tooltip.style('display','none');
    })  
    .transition(t)
    .attr('cy', (d) => y(d.life_exp))
    .attr('cx', (d) => x(d.income))
      .attr(
        'r',
        (d) =>
          Math.sqrt(area(d.population)) / Math.PI
      );

    timeLabel.text(String(time + 1800));
  }
  
  const continents = selectAll('.dropdown-item').on('click', function(e){
	let setSelection = select(this).attr('value', this.innerHTML);
  selection = setSelection.attr('value');
  update(selection);
    })

  
  function step(){
  	time = (time < 214) ? time + 1 : 0;
    update(selection);
  }
  
  const play = select('body')
  		.append('button')
  		.text('Play')
  		.attr('class', 'play-button')
  		.on('click', function(e){
        	const button = this;
        	if(button.textContent === 'Play'){
            		button.textContent = 'Pause';
          			int = setInterval(step, 100)
          }
      		else {
          		button.textContent = 'Play';
            	clearInterval(int)
          }
      })
  
  const reset = select('body')
  			.append('button')
  			.text('reset')
  			.on('click', () => {
        		time = 0; 
          	update(selection)
        })
  
  const slider = d3.sliderHorizontal()
			.min(1800)
			.max(2014)
			.step(1)
  		.width(300)
  		.tickFormat(format(''))
			.on('onchange', val => {
      		// select('#timelabel').text() - 1800;
        	timeLabel.text(val);
        	time = val;
        	console.log(time)
      })
  
  
 const sliderDiv =  select('body').append('div')
  		.attr('id', 'simple-slider')
 
  
 sliderDiv
  	.append('svg')
  	.attr('width', 500)
  	.attr('height', 100)
  .append('g')
    .attr('transform', 'translate(30,30)')
    .call(slider)
  
  update()
}


main();